/*
 *
 * $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
 *
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2006 Sebastian Trueg <trueg@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include "service.h"
#include "servicedbusinterfacebackend.h"
#include "message.h"
#include "result.h"
#include "servicedesc.h"


class Nepomuk::Backbone::Service::Private
{
public:
    DBus::ServiceBackend* backend;
    ServiceDesc desc;
};



Nepomuk::Backbone::Service::Service( const ServiceDesc& desc, QObject* parent )
    : QObject( parent ),
      d( new Private() )
{
    d->backend = new DBus::ServiceBackend( desc.url, desc.type, this );
    d->desc = desc;
}


Nepomuk::Backbone::Service::~Service()
{
    delete d->backend;
    delete d;
}


const Nepomuk::Backbone::ServiceDesc& Nepomuk::Backbone::Service::desc() const
{
    return d->desc;
}


const QString& Nepomuk::Backbone::Service::type() const
{
    return d->desc.type;
}


bool Nepomuk::Backbone::Service::isAvailable() const
{
    return d->backend->isAvailable();
}


const QString& Nepomuk::Backbone::Service::url() const
{
    return d->desc.url;
}


const QString& Nepomuk::Backbone::Service::name() const
{
    return d->desc.name;
}


Nepomuk::Backbone::Result Nepomuk::Backbone::Service::methodCall( const Nepomuk::Backbone::Message& message ) const
{
    return d->backend->methodCall( message );
}

#include "service.moc"
