/*
 *
 * $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
 *
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2006 Sebastian Trueg <trueg@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include "servicedbusinterfacebackend.h"
#include "message.h"
#include "result.h"
#include "dbustools.h"
#include "rdf/queryresulttable.h"
#include "rdf/rdfdbus.h"

#include <soprano/statement.h>

#include <QtDBus>

#include <kdebug.h>


Nepomuk::Backbone::DBus::ServiceBackend::ServiceBackend( const QString& url,
							 const QString& type,
							 QObject* parent )
    : QDBusInterface( dbusServiceFromUrl(url),
                      dbusObjectFromType(type),
                      dbusInterfaceFromType(type),
                      QDBusConnection::sessionBus(),
                      parent )
{
}


Nepomuk::Backbone::DBus::ServiceBackend::~ServiceBackend()
{
}


bool Nepomuk::Backbone::DBus::ServiceBackend::isAvailable() const
{
    // FIXME: I don't think this is sufficient
    return isValid();
}


Nepomuk::Backbone::Result Nepomuk::Backbone::DBus::ServiceBackend::methodCall( const Nepomuk::Backbone::Message& message )
{
    QDBusMessage dbusReply = callWithArgumentList( QDBus::Block,
                                                   message.method(),
                                                   message.arguments() );

    if( dbusReply.type() == QDBusMessage::ErrorMessage ) {
        kDebug(300001) << "(DBus::ServiceBackend) got error reply: " << dbusReply.errorName() << " - " << dbusReply.arguments().first().toString() << endl;
        return Result::createErrorResult( -1, dbusReply.errorName(), dbusReply.arguments().first().toString() );
    }
    else if( dbusReply.type() != QDBusMessage::ReplyMessage ) {
        kDebug(300001) << "(DBus::ServiceBackend) got invalid reply while calling method "
                       << path() + '/' + interface() + '.' + message.method() << message.arguments()
                       << "in service" << service() << ": " << dbusReply.errorName() << endl;
        // FIXME: set an error message or flag
        return Result( -1 );
    }

    // FIXME: we silently assume that the method has only a single return value

    //
    // Let QtDBus handle the simple types
    //
    if( dbusReply.arguments()[0].userType() < (int)QVariant::UserType ) {
        return Result( 0, dbusReply.arguments()[0] );
    }

    //
    // Check for the non-simple types
    //
    else {
        QDBusArgument arg = dbusReply.arguments()[0].value<QDBusArgument>();

        // FIXME: analyze the signature: [1. let QtDBus extract the simple types]
        //                               2. check for the RDF cursor types

        //
        // Check for our RDF cursor types
        //

        // FIXME: store the cursor type signatures somewhere else
        if( dbusReply.signature() == "a((isss)(isss)(isss)(isss))" ) {
            QList<Soprano::Statement> sl;
            arg >> sl;
            return Result( 0, qVariantFromValue<QList<Soprano::Statement> >( sl ) );
        }
        else if( dbusReply.signature() == "(a(s)a(a((isss))))" ) {
            RDF::QueryResultTable qr;
            arg >> qr;
            return Result( 0, qVariantFromValue<RDF::QueryResultTable>( qr ) );
        }

        //
        // Check for lists of lists of maps of ...
        //
        else {
            kDebug(300001) << "(DBus::ServiceBackend) signature: " << dbusReply.signature() << endl;
            //                               3. check for lists of lists of maps and so on
            //                               4. fail all other types
            return Result( -1 );
        }
    }
}

#include "servicedbusinterfacebackend.moc"
